import matplotlib.pyplot as plt
import os
import xarray as xr
import logging
import cartopy
from matplotlib.gridspec import GridSpec
from WCD_shared_TXp import plot_pX_cond_intrusion, plot_pX_cond_nointrusion, plot_intrusion_cond_pX, plot_compare, plot_pX_cond_intrusion_lagged, find_pX_days, compute_pX
import argparse
import yaml

logger = logging.getLogger(__name__)
FORMAT = "%(asctime)-15s %(message)s"
logging.basicConfig(format=FORMAT)
logging.getLogger().setLevel(logging.INFO)


if 'BSC_MACHINE' in os.environ and os.environ['BSC_MACHINE'] == 'mn5':
    logger.info('Setting cartopy config for {} machine'.format(os.environ['BSC_MACHINE']))
    cartopy.config['data_dir'] = '/gpfs/projects/bsc32/bsc032259/data/cartopy/'

current_dir = os.path.dirname(os.path.abspath(__file__))


def compute_TXp(cfg_path):

    with open(cfg_path, "r") as f:
        config = yaml.safe_load(f)

    period_pX = config["TXp"]["period_pX"]
    cluster_vars = config["TXp"]["cluster_vars"]
    seasons = config["TXp"]["seasons"]
    percentile = config["TXp"]["percentile"]
    
    ks = config["intrusion_finder"]["ks"]
    period_cluster = config["intrusion_finder"]["period"]
    bounds = config["intrusion_finder"]["bounds"] 
    minimum_intrusion_points_pcnt = config["intrusion_finder"]["minimum_intrusion_points_pcnt"] 
    sahara_clim_roll = config["intrusion_finder"]["sahara_clim_roll"]
    sahara_region = config["intrusion_finder"]["sahara_region"] 
    ta_inf = config["intrusion_finder"]["ta_inf"]
    ta_sup = config["intrusion_finder"]["ta_sup"]
    g_inf = config["intrusion_finder"]["g_inf"]
    g_sup = config["intrusion_finder"]["g_sup"]
    cluster_vars = config["intrusion_finder"]["cluster_vars"]

    ta_levels=f"{ta_sup.split('ta')[-1]}-{ta_inf.split('ta')[-1]}"
    gh_levels=f"{g_sup.split('g')[-1]}-{g_inf.split('g')[-1]}"
    #SHARED

    metrics = [f"p{percentile}_cond_nointrusion", f"p{percentile}_cond_intrusion", f"p{percentile}_comparison", f"1-lag-p{percentile}", f"2-lag-p{percentile}", f"3-lag-p{percentile}", f"4-lag-p{percentile}", f"5-lag-p{percentile}", f"6-lag-p{percentile}"]
    kmax=max(list(ks.values()))
    identifier=""
    compute_pX(period_pX, percentile, overwrite=False)

    if not os.path.isdir(f"{current_dir}/files/p{percentile}/"):
        os.makedirs(f"{current_dir}/files/p{percentile}/")
    for metric in metrics:
        fig = plt.figure(figsize=(16*0.95, 10*0.95), dpi=200)
        gs = GridSpec(len(seasons), kmax, figure=fig, bottom=0.05, wspace=0.01, hspace=0.01)   
        for row_id, season in enumerate(seasons):
            k=ks[season]
            logger.info(season)
            if not os.path.isfile(os.path.join(current_dir, f"files/station-tasmax-p{percentile}-7dayrolling_{period_pX}_{season}.nc")) or not os.path.isfile(os.path.join(current_dir, f"files/tasmax-p{percentile}-7dayrolling_{period_pX}_{season}.nc")):
                pX_days, pX_days_station = find_pX_days(season, period_pX, percentile)
                logger.info("extract")
            else:
                logger.info("skipping extract")
                pX_days=xr.open_dataarray(os.path.join(current_dir, f"files/tasmax-p{percentile}-7dayrolling_{period_pX}_{season}.nc"))
                pX_days_station=xr.open_dataarray(os.path.join(current_dir, f"files/station-tasmax-p{percentile}-7dayrolling_{period_pX}_{season}.nc"))
            if metric == f"p{percentile}_cond_intrusion":
                plot_pX_cond_intrusion(cluster_vars, pX_days, pX_days_station, k, period_pX, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels)
            elif metric == f"p{percentile}_cond_nointrusion":
                plot_pX_cond_nointrusion(cluster_vars, pX_days, pX_days_station, k, period_pX, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels)
            elif metric == f"intrusion_p{percentile}_cond":
                plot_intrusion_cond_pX(cluster_vars, pX_days, pX_days_station, k, period_pX, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels)
            elif metric==f"p{percentile}_comparison":
                plot_compare(cluster_vars, k, period_pX, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, percentile, ta_levels, gh_levels)   
            elif f"-lag" in metric:
                plot_pX_cond_intrusion_lagged(cluster_vars, pX_days, k, period_pX, period_cluster, bounds, minimum_intrusion_points_pcnt, identifier, season, row_id, fig, gs, sahara_clim_roll, sahara_region, int(metric.split("-")[0]), percentile, ta_levels, gh_levels)
                    
        plt.tight_layout()
        plt.savefig(os.path.join(current_dir, f"plots/{metric}_{period_pX}_{'.'.join([str(ee) for ee in bounds])}_intrusions-clim-dates{identifier}_{period_cluster.replace('-','').replace('.','-')}_min-intr-points-{minimum_intrusion_points_pcnt}_sahara-{sahara_region[0]}.{sahara_region[1]}.{sahara_region[2]}.{sahara_region[3]}-roll.{sahara_clim_roll}-thresh_ta{ta_levels}_gh{gh_levels}.png"), bbox_inches="tight")
        plt.show()


if __name__ == '__main__':
    # Create an argument parser
    parser = argparse.ArgumentParser(description="")

    DEFAULT_FILE_PATH = os.path.join(current_dir,"config_file.yml")
    parser.add_argument("--config_file", "-f", type=str, default=DEFAULT_FILE_PATH, help="Path to the config file")

    args = parser.parse_args()

    # Validate that the file exists
    if not os.path.isfile(args.config_file):
        print(f"Error: The file '{args.config_file}' does not exist.")
        exit(1)

    compute_TXp(args.config_file)

